/*
# startup_LPC17xx.c		V.0.1
#
# Copyright (C) 2011 audin.
# http://avr.paslog.jp
# This file is licensed under the MIT License.
#
# 2011/04  V.0.1:	Modified and translated from 32bitmicro's startup_LPC17xx.S to C language.
#					( It is referenced to http://code.google.com/p/32bitmicro/ )
#					And also referenced ARM Limited (ARM)'s startup_LPC17xx.c from
#					eLua project's LPC1786 port.
#					( It is referenced to http://www.eluaproject.net/ )
*/


/*
 * 32bitmicro startup_LPC17xx.S  Startup File for the NXP LPC17xx Device Series 

 * $URL$
 * $ID$

Copyright (c) 2009, 32bitmicro
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.
Neither the name of the 32bitmicro nor the names of its contributors may be used to endorse or promote products derived from this software without 
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "LPC17xx.h"

#if defined (__cplusplus)
	extern "C" {
#endif


#define WEAK __attribute__ ((weak))

/* Exception vector handler */
void WEAK	Reset_Handler(void);		/* Reset Handler					*/
void WEAK	NMI_Handler(void);			/* NMI Handler						*/
void WEAK	HardFault_Handler(void);	/* Hard Fault Handler				*/
void WEAK	MemManage_Handler(void);	/* MPU Fault Handler				*/
void WEAK	BusFault_Handler(void);		/* Bus Fault Handler				*/
void WEAK	UsageFault_Handler(void);	/* Usage Fault Handler				*/
void WEAK	SVC_Handler(void);			/* SVCall Handler					*/
void WEAK	DebugMon_Handler(void);		/* Debug Monitor Handler			*/
void WEAK	PendSV_Handler(void);		/* PendSV Handler					*/
void WEAK	SysTick_Handler(void);		/* SysTick Handler 					*/

/* Interrupt vector handler */
void WEAK	WDT_IRQHandler(void);		/* 16: Watchdog Timer				*/
void WEAK	TIMER0_IRQHandler(void);	/* 17: Timer0						*/
void WEAK	TIMER1_IRQHandler(void);	/* 18: Timer1						*/
void WEAK	TIMER2_IRQHandler(void);	/* 19: Timer2						*/
void WEAK	TIMER3_IRQHandler(void);	/* 20: Timer3						*/
void WEAK	UART0_IRQHandler(void);		/* 21: UART0						*/
void WEAK	UART1_IRQHandler(void);		/* 22: UART1						*/
void WEAK	UART2_IRQHandler(void);		/* 23: UART2						*/
void WEAK	UART3_IRQHandler(void);		/* 24: UART3						*/
void WEAK	PWM1_IRQHandler(void);		/* 25: PWM1							*/
void WEAK	I2C0_IRQHandler(void);		/* 26: I2C0							*/
void WEAK	I2C1_IRQHandler(void);		/* 27: I2C1							*/
void WEAK	I2C2_IRQHandler(void);		/* 28: I2C2							*/
void WEAK	SPI_IRQHandler(void);		/* 29: SPI							*/
void WEAK	SSP0_IRQHandler(void);		/* 30: SSP0							*/
void WEAK	SSP1_IRQHandler(void);		/* 31: SSP1							*/
void WEAK	PLL0_IRQHandler(void);		/* 32: PLL0 Lock (Main PLL)			*/
void WEAK	RTC_IRQHandler(void);		/* 33: Real Time Clock				*/
void WEAK	EINT0_IRQHandler(void);		/* 34: External Interrupt 0			*/
void WEAK	EINT1_IRQHandler(void);		/* 35: External Interrupt 1			*/
void WEAK	EINT2_IRQHandler(void);		/* 36: External Interrupt 2			*/
void WEAK	EINT3_IRQHandler(void);		/* 37: External Interrupt 3			*/
void WEAK	ADC_IRQHandler(void);		/* 38: A/D Converter				*/
void WEAK	BOD_IRQHandler(void);		/* 39: Brown-Out Detect				*/
void WEAK	USB_IRQHandler(void);		/* 40: USB							*/
void WEAK	CAN_IRQHandler(void);		/* 41: CAN							*/
void WEAK	DMA_IRQHandler(void);		/* 42: General Purpose DMA			*/
void WEAK	I2S_IRQHandler(void);		/* 43: I2S							*/
void WEAK	ENET_IRQHandler(void);		/* 44: Ethernet						*/
void WEAK	RIT_IRQHandler(void);		/* 45: Repetitive Interrupt Timer	*/
void WEAK	MCPWM_IRQHandler(void);		/* 46: Motor Control PWM			*/
void WEAK	QEI_IRQHandler(void);		/* 47: Quadrature Encoder Interface	*/
void WEAK	PLL1_IRQHandler(void);		/* 48: PLL1 Lock (USB PLL)			*/

/*******************************
* external functions
 ********************************/
void ResetISR(void) __attribute__((__interrupt__));
int   main(void);
int __main(void);
void _vStackTop(void);	/* init value for the stack pointer. defined in linker script */

/*******************************
* external function for FreeRTOS
 ********************************/
#if defined( __FREE_RTOS__ )
void vPortSVCHandler(void);
void xPortPendSVHandler(void);
void xPortSysTickHandler(void);
#endif

#if defined ( __cplusplus )
} 
#endif	/*  extern "C" */

/*******************************
* external varuables
 ********************************/
extern unsigned long _etext;	/* end of text section */
extern unsigned long _data;		/* start address for the initialization values of the .data section. defined in linker script */
extern unsigned long _edata;	/* end address for the .data section. defined in linker script */
extern unsigned long _bss;		/* start address for the .bss section. defined in linker script */
extern unsigned long _ebss;		/* end address for the .bss section. defined in linker script */

/*******************************
* Vector table
 ********************************/
__attribute__ ((section(".isr_vector")))
void (* const g_pfnVectors[])(void) =
{
  &_vStackTop,							/* 0: Top of Stack					*/
  /* CM3 system interrupts */
  ResetISR,								/* 1: Reset Handler					*/
  NMI_Handler,							/* 2: NMI Handler					*/
  HardFault_Handler,					/* 3: Hard Fault Handler			*/
  MemManage_Handler,					/* 4: MPU Fault Handler				*/
  BusFault_Handler,						/* 5: Bus Fault Handler				*/
  UsageFault_Handler,					/* 6: Usage Fault Handler			*/
  0,									/* 7: Reserved						*/
  0,									/* 8: Reserved						*/
  0,									/* 9: Reserved						*/
  0,									/* 10: Reserved						*/

#if defined( __FREE_RTOS__ )
  vPortSVCHandler,						/* 11: SVCall Handler				*/
#else
  SVC_Handler,							/* 11: SVCall Handler				*/
#endif
  DebugMon_Handler,						/* 12: Debug Monitor Handler		*/
  0,									/* 13: Reserved						*/
#if defined( __FREE_RTOS__ )
  xPortPendSVHandler,					/* 14: PendSV Handler				*/
  xPortSysTickHandler,					/* 15: SysTick Handler				*/
#else
  PendSV_Handler,						/* 14: PendSV Handler				*/
  SysTick_Handler,	 					/* 15: SysTick Handler				*/
#endif

/*******************************
* LPC17xx interrupts
 ********************************/
  WDT_IRQHandler,						/* 16: Watchdog Timer				*/
  TIMER0_IRQHandler,					/* 17: Timer0						*/
  TIMER1_IRQHandler,					/* 18: Timer1						*/
  TIMER2_IRQHandler,					/* 19: Timer2						*/
  TIMER3_IRQHandler,					/* 20: Timer3						*/
  UART0_IRQHandler,						/* 21: UART0						*/
  UART1_IRQHandler,						/* 22: UART1						*/
  UART2_IRQHandler,						/* 23: UART2						*/
  UART3_IRQHandler,						/* 24: UART3						*/
  PWM1_IRQHandler,						/* 25: PWM1							*/
  I2C0_IRQHandler,						/* 26: I2C0							*/
  I2C1_IRQHandler,						/* 27: I2C1							*/
  I2C2_IRQHandler,						/* 28: I2C2							*/
  SPI_IRQHandler,						/* 29: SPI							*/
  SSP0_IRQHandler,						/* 30: SSP0							*/
  SSP1_IRQHandler,						/* 31: SSP1							*/
  PLL0_IRQHandler,						/* 32: PLL0 Lock (Main PLL)			*/
  RTC_IRQHandler,						/* 33: Real Time Clock				*/
  EINT0_IRQHandler,						/* 34: External Interrupt 0			*/
  EINT1_IRQHandler,						/* 35: External Interrupt 1			*/
  EINT2_IRQHandler,						/* 36: External Interrupt 2			*/
  EINT3_IRQHandler,						/* 37: External Interrupt 3			*/
  ADC_IRQHandler,						/* 38: A/D Converter				*/
  BOD_IRQHandler,						/* 39: Brown-Out Detect				*/
  USB_IRQHandler,						/* 40: USB							*/
  CAN_IRQHandler,						/* 41: CAN							*/
  DMA_IRQHandler,						/* 42: General Purpose DMA			*/
  I2S_IRQHandler,						/* 43: I2S							*/
  ENET_IRQHandler,						/* 44: Ethernet						*/
  RIT_IRQHandler,						/* 45: Repetitive Interrupt Timer	*/
  MCPWM_IRQHandler,						/* 46: Motor Control PWM			*/
  QEI_IRQHandler,						/* 47: Quadrature Encoder Interface	*/
  PLL1_IRQHandler						/* 48: PLL1 Lock (USB PLL)			*/
  };

/*******************************
* ResetISR : Reset handler
 ********************************/
void ResetISR(void)
{
  unsigned long *pulSrc, *pulDest;

  /* Copy initialized .data section from rom to ram */
  pulSrc = &_etext;
  for( pulDest = &_data; pulDest < &_edata; ){
	*( pulDest++ ) = *( pulSrc++ );
  }

  /* Zero fill to .bss section */
  for( pulDest = &_bss; pulDest < &_ebss; ){
	*( pulDest++ ) = 0;
  }

  /* Call PLL settings */
  SystemInit();

#if defined (__cplusplus)
  /*  Call C++ library initialization */
  __libc_init_array();
#endif

  /* Call main program */
#if defined (__REDLIB__)
  __main() ;
#else
  main();
#endif

  while(1){
	/* Terminate */
  }
}

/* Weak aliases to  Exception vector handler */
#pragma weak MemManage_Handler = Default_Handler	/* MPU Fault Handler				*/
#pragma weak BusFault_Handler = Default_Handler		/* Bus Fault Handler				*/
#pragma weak UsageFault_Handler = Default_Handler	/* Usage Fault Handler				*/
#pragma weak SVC_Handler = Default_Handler			/* SVCall Handler					*/
#pragma weak DebugMon_Handler = Default_Handler		/* Debug Monitor Handler			*/
#pragma weak PendSV_Handler = Default_Handler		/* PendSV Handler					*/
#pragma weak SysTick_Handler = Default_Handler		/* SysTick Handler					*/

/* Weak aliases to Interrupt vector handler */
#pragma weak WDT_IRQHandler = Default_Handler		/* 16: Watchdog Timer				*/
#pragma weak TIMER0_IRQHandler = Default_Handler	/* 17: Timer0						*/
#pragma weak TIMER1_IRQHandler = Default_Handler	/* 18: Timer1						*/
#pragma weak TIMER2_IRQHandler = Default_Handler	/* 19: Timer2						*/
#pragma weak TIMER3_IRQHandler = Default_Handler	/* 20: Timer3						*/
#pragma weak UART0_IRQHandler = Default_Handler		/* 21: UART0						*/
#pragma weak UART1_IRQHandler = Default_Handler		/* 22: UART1						*/
#pragma weak UART2_IRQHandler = Default_Handler		/* 23: UART2						*/
#pragma weak UART3_IRQHandler = Default_Handler		/* 24: UART3						*/
#pragma weak PWM1_IRQHandler = Default_Handler		/* 25: PWM1							*/
#pragma weak I2C0_IRQHandler = Default_Handler		/* 26: I2C0							*/
#pragma weak I2C1_IRQHandler = Default_Handler		/* 27: I2C1							*/
#pragma weak I2C2_IRQHandler = Default_Handler		/* 28: I2C2							*/
#pragma weak SPI_IRQHandler = Default_Handler		/* 29: SPI							*/
#pragma weak SSP0_IRQHandler = Default_Handler		/* 30: SSP0							*/
#pragma weak SSP1_IRQHandler = Default_Handler		/* 31: SSP1							*/
#pragma weak PLL0_IRQHandler = Default_Handler		/* 32: PLL0 Lock (Main PLL)			*/
#pragma weak RTC_IRQHandler = Default_Handler		/* 33: Real Time Clock				*/
#pragma weak EINT0_IRQHandler = Default_Handler		/* 34: External Interrupt 0			*/
#pragma weak EINT1_IRQHandler = Default_Handler		/* 35: External Interrupt 1			*/
#pragma weak EINT2_IRQHandler = Default_Handler		/* 36: External Interrupt 2			*/
#pragma weak EINT3_IRQHandler = Default_Handler		/* 37: External Interrupt 3			*/
#pragma weak ADC_IRQHandler = Default_Handler		/* 38: A/D Converter				*/
#pragma weak BOD_IRQHandler = Default_Handler		/* 39: Brown-Out Detect				*/
#pragma weak USB_IRQHandler = Default_Handler		/* 40: USB							*/
#pragma weak CAN_IRQHandler = Default_Handler		/* 41: CAN							*/
#pragma weak DMA_IRQHandler = Default_Handler		/* 42: General Purpose DMA			*/
#pragma weak I2S_IRQHandler = Default_Handler		/* 43: I2S							*/
#pragma weak ENET_IRQHandler = Default_Handler		/* 44: Ethernet						*/
#pragma weak RIT_IRQHandler = Default_Handler		/* 45: Repetitive Interrupt Timer	*/
#pragma weak MCPWM_IRQHandler = Default_Handler		/* 46: Motor Control PWM			*/
#pragma weak QEI_IRQHandler = Default_Handler		/* 47: Quadrature Encoder Interface	*/
#pragma weak PLL1_IRQHandler = Default_Handler		/* 48: PLL1 Lock (USB PLL)			*/

/****************
* Default handlers
 *****************/
void Default_Handler(void)
{
  /* Go into an infinite loop. */
  while (1) {
	/* no work */
  }
}

/*********************
* Code read protection 
 **********************/
/* Don't change value of __CRP_Key constant if you don't know about specification of C.R.P. */
#define CRP_SECTION	".CRP._0x02FC"	// ".crp"
__attribute__ ( ( section( CRP_SECTION ) ) ) uint32_t __CRP_Key = 0xFFFFFFFF;

	 
/*
#########################################################################
# References:
# NXP CMSIS/etc page 
# http://ics.nxp.com/support/documents/microcontrollers/?sort=date#All

# Direct link for CMSIS Drivers lpc17xx
# http://ics.nxp.com/support/documents/microcontrollers/zip/lpc17xx.cmsis.driver.library.zip


#########################################################################

# The MIT License
# 
# Copyright (c) 2011 audin.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
*/
